#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMessageBox>
#include <QList>

#include "keyboard.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    loaded = false;
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_load_clicked()
{
    setButtons(false);
    Thread *thread = new Thread(this, &Thread::read);
    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
    thread->start();
}

void MainWindow::on_store_clicked()
{
    setButtons(false);
    Thread *thread = new Thread(this, &Thread::write);
    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
    thread->start();
}

void MainWindow::on_info_clicked()
{
    setButtons(false);
    Thread *thread = new Thread(this, &Thread::information);
    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
    thread->start();
}

void MainWindow::on_calibrate_clicked()
{
    setButtons(false);
    Thread *thread = new Thread(this, &Thread::calibrate);
    connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
    thread->start();
}

void MainWindow::on_en_clicked()
{
    ui->cz->setChecked(false);
    ui->en->setChecked(true);
}

void MainWindow::on_cz_clicked()
{
    ui->cz->setChecked(true);
    ui->en->setChecked(false);
}

void MainWindow::on_setText(const QString text, bool cz)
{
    loaded = true;
    ui->text->setEnabled(true);
    ui->text->setPlainText(text);
    ui->cz->setChecked(cz);
    ui->en->setChecked(!cz);
}

void MainWindow::on_error(const QString text)
{
    QMessageBox::critical(this, "Error", text);
    setButtons(true);
}

void MainWindow::on_warning(const QString text)
{
    QMessageBox::warning(this, "Warning", text);
    setButtons(true);
}

void MainWindow::on_info(const QString text)
{
    QMessageBox::information(this, "Success", text);
    setButtons(true);
}

void MainWindow::setButtons(bool s)
{
    ui->load->setEnabled(s);
    ui->store->setEnabled(s & loaded);
    ui->info->setEnabled(s);
    ui->calibrate->setEnabled(s);
}

Thread::Thread(MainWindow *mw, void (Thread::*f)(protocol &p)) : mw(mw), f(f)
{
    connect(this, SIGNAL(setText(const QString, bool)), mw, SLOT(on_setText(const QString, bool)));
    connect(this, SIGNAL(error(const QString)), mw, SLOT(on_error(const QString)));
    connect(this, SIGNAL(warning(const QString)), mw, SLOT(on_warning(const QString)));
    connect(this, SIGNAL(info(const QString)), mw, SLOT(on_info(const QString)));
}

Thread::~Thread()
{
    disconnect(this, SIGNAL(setText(const QString, bool)), mw, SLOT(on_setText(const QString, bool)));
    disconnect(this, SIGNAL(error(const QString)), mw, SLOT(on_error(const QString)));
    disconnect(this, SIGNAL(warning(const QString)), mw, SLOT(on_warning(const QString)));
    disconnect(this, SIGNAL(info(const QString)), mw, SLOT(on_info(const QString)));
}

void Thread::run()
{
    if (hid_init()) {
        emit error("HIDAPI could not be initialized!");
        return;
    }

    struct hid_device_info *devs = hid_enumerate(0x16C0, 0x05DF);
    struct hid_device_info *cur_dev = devs;
    char *path = NULL;
    while (cur_dev) {
        QString name;
        wchar_t *c = cur_dev->product_string;
        while (*c != 0)
            name += *(c++);
        if (name == "PassKeychain") {
            path = (char*) malloc(sizeof(char)*(strlen(cur_dev->path)+1));
            strcpy(path, cur_dev->path);
            break;
        }
        cur_dev = cur_dev->next;
    }
    hid_free_enumeration(devs);

    if (!path) {
        emit error("No Passkeychain found!");
        return;
    }

    hid_device *handle = hid_open_path(path);
    if (!handle) {
        emit error("Unable to open PassKeychain device!");
        free(path);
        return;
    }

    protocol p(handle);
    p.clear();
    (this->*f)(p);

    hid_close(handle);
    hid_exit();
    free(path);
}

void Thread::read(protocol &protocol)
{
    QString pswds;
    QByteArray data[3];
    // Get
    for (int i = 0; i < 3; ++i) {
        data[i] = protocol.read_chunk(i*170+10, 160);
        // Check size
        if (data[i].size() != 160) {
            QString e; e.sprintf("Only %d/160 bytes read.", data[i].size());
            emit warning(e);
            return;
        }
        // Decompress
        if (!decompress(data[i], pswds)) {
            emit info("Data successfuly loaded");
            return;
        }
    }
    // Try to combine
    for (int i = 0; i < 160; ++i) {
        if (data[0][i] != data[1][i]) {
            if (data[0][i] == data[2][i]) data[1][i] = data[2][i];
            else if (data[1][i] == data[2][i]) data[0][i] = data[2][i];
        } else if (data[1][i] != data[2][i] && data[1][i] == data[0][i])
            data[2][i] = data[0][i];
    }
    // Decompress again?
    pswds.clear();
    for (int i = 0; i < 3; ++i) {
        // Decompress
        if (!decompress(data[i], pswds)) {
            emit info("Data successfuly recovered :)");
            return;
        }
    }
    emit setText(pswds, false);
    emit warning("Data probably lost :'(");
}

void Thread::write(protocol &protocol)
{
    bool cz = mw->ui->cz->isChecked();
    QString text = mw->ui->text->toPlainText();
    QList<QString> passwords;
    QList<QByteArray> keys;
    QByteArray data;
    QString line;

    // Split
    for (int z = 0; z < text.size(); ++z) {
        QChar c  = text.at(z);
        if (c.isPrint()) {
            line += c;
        } else if (c != '\t') {
            if (line.size())
                passwords.append(line);
            line.clear();
        }
    }
    if (line.size())
        passwords.append(line);

    // Convert
    for (int z = 0; z < passwords.size(); ++z) {
        QString s = passwords.at(z);
        QByteArray b = strToKeys(s, mw->ui->cz->isChecked());
        if (b.isEmpty()) {
            emit warning("Could not convert:\n"+s);
            return;
        }
        keys.append(b);
    }

    // Header
    int crc = keys.size()+0x80*cz;
    data.append(crc);
    data.append(~crc);
    for (int z = 0; z < keys.size(); ++z) {
        QByteArray b = keys.at(z);
        data.append(b.size()+1);
        crc += b.size()+1;
    }
    data.append(crc);

    // Passwords
    for (int z = 0; z < keys.size(); ++z) {
        QByteArray b = keys.at(z);
        crc = 0;
        for (int z = 0; z < b.size(); ++z) {
            unsigned char c = b.at(z);
            data.append(c);
            crc += c;
        }
        data.append(crc);
    }

    // Check
    if (data.length() > 160) {
        QString s;
        s.sprintf("Data too large: %d\nmust be 160 at maximum", data.length());
        emit warning(s);
        return;
    }

    // Store into 3 identical blocks
    if (!protocol.write_chunk(data, 10)) {
        emit warning("Could not write data at position 10");
        return;
    }
    if (!protocol.write_chunk(data, 180)) {
        emit warning("Could not write data at position 180");
        return;
    }
    if (!protocol.write_chunk(data, 350)) {
        emit warning("Could not write data at position 350");
        return;
    }

    QString s;
    s.sprintf("Successfully stored %d bytes", data.length());
    emit info(s);
}

void Thread::information(protocol &protocol)
{
    QString s, b;
    s += "ID: "+protocol.getID()+"\n";
    s += "OSCCAL: "+protocol.getOSSCAL()+"\n";
    QByteArray data = protocol.read_chunk(0, 512);
    int i = 0;
    for (int z = 0; z < data.size(); ++z) {
        unsigned char c = data.at(z);
        b += QString::number(c>>4, 16) + QString::number(c&0xf, 16);
        if (++i%16) b += " ";
        else b += "\n";
    }
    emit setText(b, false);
    s += "Received "+QString::number(data.size())+" bytes of memory";
    emit info(s);
}

void Thread::calibrate(protocol &protocol)
{
    protocol.calibrate();
    emit info("Calibration request sent");
}

int Thread::decompress(QByteArray &data, QString &append) {
    // Header
    int n = (unsigned char)(data.at(0)&0x7f);
    bool cz = data.at(0)>>7;
    int crc = n+0x80*cz;
    if ((unsigned char)crc != (unsigned char)data.at(0))
        return 1;
    if ((unsigned char)crc != (unsigned char)(~data.at(1)))
        return 2;

    // Frames
    int offset = 3+n;
    QList<int> addr;
    for (int i = 0; i < n; ++i) {
        addr.append(offset);
        offset += (unsigned char)data.at(2+i);
        crc += (unsigned char)data.at(2+i);
    }
    if ((unsigned char)crc != (unsigned char)data.at(2+n))
        return 4;

    // Keys
    bool ok = true;
    QString text;
    for (int i = 0; i < n; ++i) {
        crc = 0;
        QByteArray key;
        int len = (unsigned char)data.at(2+i)-1;
        for (int j = 0; j < len; ++j) {
            key.append(data.at(addr.at(i)+j));
            crc += data.at(addr.at(i)+j);
        }
        QString s = keysToStr(key, cz);
        if ((unsigned char)crc == (unsigned char)data.at(addr.at(i)+len) && !s.isEmpty()) {
            text += s + '\n';
            append += s + '\n';
        } else {
            ok = false;
            append += "( "+s+" )\n";
        }
    }
    if (!ok)
        return 5;
    emit setText(text, cz);
    return 0;
}
