#include "protocol.h"

#ifdef Q_OS_WIN
#include <windows.h>
#endif

protocol::protocol(hid_device *handle)
    : device(handle)
{
}

void protocol::qSleep(int ms)
{
#ifdef Q_OS_WIN
    Sleep(uint(ms));
#else
    struct timespec ts = { ms / 1000, (ms % 1000) * 1000 * 1000 };
    nanosleep(&ts, NULL);
#endif
}

QByteArray protocol::hid_read_(void)
{
    QByteArray array;
    unsigned char buffer[8];
    int res = 0;
    do {
        for (int i = 0; i < 5; ++i) {
            buffer[0] = 0x01;
            if ((res = hid_get_feature_report(device, buffer, 9)) > 1)
                break;
            qSleep(10);
        }
#if defined(Q_OS_WIN)
            --res;
#endif
        for (int i = 1; i < res; ++i)
            array.append(buffer[i]);
    } while (res > 1);
    return array;
}

bool protocol::hid_write_(const QByteArray &data)
{
    int buffer_size = 0;
    unsigned char buffer[9];
    for (int i = 0;;) {
        buffer[++buffer_size] = data.at(i++);
        if (buffer_size == 8 || i == data.size()) {
            buffer[0] = 0x1;
            while (buffer_size < 8)
                buffer[++buffer_size] = 36;
            // do not check the send state
            hid_send_feature_report(device, buffer, 9);
            if (i == data.size())
                return true;
            buffer_size = 0;
        }
    }
}

void protocol::clear()
{
    QByteArray b;
    for (int i = 0; i < 8; ++i) b.append(32);
    hid_write_(b);
    hid_read_();
}

QString protocol::getID(void)
{
    QByteArray a;
    for (int i = 0; i < 7; ++i)
        a.append(36);
    a.append(63);
    hid_write_(a);
    a = hid_read_();
    if (a.size() < 4 || a.at(0) != 36 ||
        a.at(1) != 33 || (unsigned char)a.at(2)+4 != a.size())
        return QString("frame error");
    int crc = 29+a.size();
    QString s;
    for (int i = 3; i < (unsigned char)a.size()-1; ++i) {
        s += a.at(i);
        crc += (unsigned char)a.at(i);
    }
    if ((unsigned char)crc != (unsigned char)a.at(a.size()-1))
        return QString("crc error");
    return s;
}

QString protocol::getOSSCAL(void)
{
    QByteArray a;
    a.append(36);
    a.append('>');
    a.append(1);
    a.append(1+'>');
    hid_write_(a);
    a = hid_read_();
    if (a.size() < 4 || a.at(0) != 36 ||
        a.at(1) != '@' || a.at(2) != 1 || 5 != a.size())
        return QString("frame error");
    int crc = '@'+1+a.at(3);
    if ((unsigned char)crc != (unsigned char)a.at(4))
        return QString("crc error");
    return QString::number((unsigned char)a.at(3))+" (0x"+
           QString::number((a.at(3)>>4)&0xf, 16)+
           QString::number(a.at(3)&0xf, 16)+")";
}

void protocol::calibrate(void)
{
    QByteArray a;
    a.append(36);
    a.append('^');
    a.append(0x2b);
    a.append(0x95);
    a.append((char)('^'+0x2b+0x95));
    hid_write_(a);
}

QByteArray protocol::read_chunk(int addr, int len)
{
    int total_len = len;
    QByteArray data;
    while (data.size() < total_len && addr+len <= 512) {
        int l = std::min(len, 16);
        int crc = l+(unsigned char)(addr&0xff)+(unsigned char)((addr>>8)&0xff);
        QByteArray request;
        request.append('$');
        request.append('<');
        request.append(l);
        request.append(addr&0xff);
        request.append((addr>>8)&0xff);
        request.append('<'+crc);
        for (int t = 0;;) {
            if (++t > 3)
                return data;
            if (!hid_write_(request)) {
                clear();
                continue;
            }
            QByteArray response = hid_read_();
            if (response.length() != l+6 || response.at(0) != 36 ||
                response.at(1) != '=' || response.at(2) != 2+l ||
                (unsigned char)response.at(3) != (unsigned char)(addr&0xff) ||
                (unsigned char)response.at(4) != (unsigned char)((addr>>8)&0xff)) {
                clear();
                continue;
            }
            int c = crc;
            for (int i = 0; i < l; ++i)
                c += (unsigned char)(response.at(i+5));
            if ((unsigned char)response.at(response.size()-1) != (unsigned char)(c+2+'=')) {
                clear();
                continue;
            }
            for (int i = 0; i < l; ++i)
                data.append(response.at(i+5));
            break;
        }
        addr += l;
        len -= l;
    }
    return data;
}

bool protocol::write_chunk(const QByteArray &data, int addr)
{
    int index = 0;
    const int data_length = 2;
    while (index < data.size() && addr+data.size()-index <= 512) {
        int l = std::min(data.size()-index, data_length);
        int crc = (unsigned char)(addr&0xff)+(unsigned char)((addr>>8)&0xff);
        QByteArray request;
        request.append('$');
        request.append('#');
        request.append(l+data_length);
        request.append(addr&0xff);
        request.append((addr>>8)&0xff);
        for (int i = 0; i < l; ++i) {
            request.append(data.at(index+i));
            crc += (unsigned char)data.at(index+i);
        }
        request.append(crc+l+data_length+'#');
        for (int t = 0;;) {
            if (++t > 3)
                return false;
            if (!hid_write_(request)) {
                clear();
                continue;
            }
            QByteArray response = hid_read_();
            if (response.length() != 2 || response.at(0) != 36 || response.at(1) != 48) {
                clear();
                continue;
            }
            break;
        }
        addr += l;
        index += l;
    }
    return true;
}
