/*
 * Name: HW.c
 * Project: PassKeychain
 * Author: Jiri Stepanovsky - http://jiristepanovsky.cz
 * Creation Date: 2014-11-24
 * Tabsize: 4
 * License: MIT License
 */

/* Cykles count => time */
#define FIVESECONDS 170000  // Around 5 sec (32-bit)
#define PRESTIMEOUT 50000   // Around 1.1 sec (16-bit)
#define ERRORFILTER 10      // Around 0.0 sec (8-bit)

/* include */
#include <avr/io.h>
#include "HW.h"

/* State of led: 0-OFF, 1-ON, 2-BLINK, 3-PWM, 4-PWMshading */
unsigned char led_state;
/* Pwm/Time values used to control led (blink with led and other effects) */
unsigned char led_value, led_pwm_counter, led_dirr;
unsigned short led_counter; // delay in timer mode

/* Button variables used to find out state of button (pressed/released) */
unsigned char button_state, button_delay, button_press_count, button_return_value;
unsigned short button_press_delay; // time of released button
unsigned long button_5S_delay; // total time of pressed button

/* Initialize hardware */
unsigned char HWinit(void){

    // Clear led variables
	led_state = 0;
	button_state = 100;
	button_5S_delay = 0;
	button_press_count = 0;
	button_press_delay = 0;

	// LED as output
	DDRB |= 0x10; // 00010000
	
	// LED off, button pull-up
	PORTB |= 0x08; // 00001000

	// return button press state
	for(button_delay = 0; button_delay < 20; button_delay++){
		if(PINB & 0x08){
			button_state--; // button released
		} else {
			button_state++; // button pressed
		}
	}
	button_delay = 0;
	if(button_state > 110){
		button_state = 1; // value for pressed button
		return 1;
	}
	button_state = 0; // value for released button
	return 0;

}

/* Reset I/O configuration */
void HWreset(void){
    
    // LED as output
	DDRB |= 0x10; // 00010000
	
	// LED off, button pull-up
	PORTB |= 0x08; // 00001000

    // led state = 0 (OFF)
    led_state = 0;
}

/* Poll everything, returns: 0-No event, 1~254-Number of clicks, 255-5 sec button press */
unsigned char HWpoll(void){

	// Poll led
	if(led_state == 2){
		// led in timer mode
		led_counter++;
		if( ((led_counter>>8)&0xFF) == led_value){
			led_counter = 0;
			PORTB ^= 0x10; // XOR LED pin
		}
	} else if(led_state == 3){
		// led in pwm mode
		if (!led_pwm_counter && led_value){
			PORTB |= 0x10; // LED on
		}
		if( led_pwm_counter == led_value && led_value < 0xFF){
			PORTB &= 0xEF; // LED off
		}
		led_pwm_counter++;
	} else if(led_state == 4){
		// led in pwm shading mode
		if (!led_pwm_counter && led_value){
			PORTB |= 0x10; // LED on
		}
		if( led_pwm_counter == led_value && led_value < 0xFF){
			PORTB &= 0xEF; // LED off
		}
		// shading routine
		led_pwm_counter++;
		if(!led_pwm_counter){
			if(led_dirr){
				led_value++;
				if(led_value == 0xFF)
					led_dirr = 0;
			} else {
				led_value--;
				if(led_value == 0)
					led_dirr = 1;
			}
		}
	}

	// Null return value
	if(button_return_value)
		button_return_value = 0;

	// Button poll
	if(button_state){
		// Button is pressed

		// Counter for 5s press
		if(button_5S_delay < FIVESECONDS){
			button_5S_delay++;
		}

		if(PINB & 0x08){
			// Button released
			button_delay++;
			if(button_delay > ERRORFILTER){
                // Button actualy released
				button_state = 0;
				button_5S_delay = 0;
				button_delay = 0;
			}
		} else {
			// Button still pressed
			button_delay = 0;
		}
	} else {
		// Button is released

		// Check delay between clicks
		button_press_delay++;
		if(button_press_delay > PRESTIMEOUT){
			// Delay is too long
			button_press_delay = 0;
			if(button_press_count){
				// Repeated pressing of button ended - return number of clicks
				button_return_value = button_press_count;
				button_press_count = 0;
			}
		}

        // Check button
		if(PINB & 0x08){
			// Button still released
			button_delay = 0;
		} else {
			// Button pressed
			button_delay++;
			if(button_delay > ERRORFILTER){
                // Button actualy pressed - increase number of clicks
				if(button_press_count < 254)
					button_press_count++;
				button_press_delay = 0;
				button_state = 1;
				button_delay = 0;
			}
		}
	}
		
	// Button has been pressed for more then 5s
	if(button_5S_delay == FIVESECONDS){
		button_5S_delay++;
		button_press_count = 0; // Null number of button clicks from history
		return 0xFF;
	}

	// Button has been multi-times pressed
	if(button_return_value)
		return button_return_value; // return 1 - 254
	
    // Nothing happend
	return 0;
}

/* Set led state 0 (OFF) */
void led_off(void){
	led_state = 0;
	PORTB &= 0xEF; // 11101111
}

/* Set led state 1 (ON) */
void led_on(void){
	led_state = 1;
	PORTB |= 0x10; // 00010000
}

/* Set led state 2 (BLINK) */
void led_timer(unsigned char time){
	led_state = 2;
	led_value = time;
	led_counter = 0;
}

/* Set led state 3 (PWM)*/
void led_pwm(unsigned char pwm){
	led_state = 3;
	led_value = pwm;
	led_pwm_counter = 0;
}

/* Set led state 4 (PWM-SHADING)*/
void led_shade(void){
	if(led_state < 3){
		if(PORTB & 0x10){
			led_value = 255;
			led_dirr = 0;
		} else {
			led_value = 0;
			led_dirr = 1;
		}
	} else {
		if(led_value < 128){
			led_dirr = 1;
		} else {
			led_dirr = 0;
		}
	}
	led_state = 4;
	led_pwm_counter = 0;
}
