/* 
 * Name: main.c
 * Project: PassKeychain
 * Author: Jiri Stepanovsky - http://jiristepanovsky.cz
 * Creation Date: 2020-11-11
 * License: MIT License
 */

#define F_CPU 16500000

#include <string.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#include "usbdrv_1.h"		// Keyboard
#include "usbdrv_2.h"		// Hid specific
//#include "oddebug.h"
#include "buffer.h"
#include "HW.h"


/*------------------------------------- COMMON ---------------------------------------*/

// Common
static uchar size, i;

// Led
static ushort ledProcess;
#define HID_OK led_on();
#define HID_BUSY led_timer(20);
#define KEYBOARD_OK led_shade();
#define KEYBOARD_BUSY led_timer(20);

// Hid specific
uchar receivedByte = 0;
uchar processedCommand = 0;
uchar commandReceivedByte = 0;
uchar CRCbyte = 0;
uchar len = 0;

// Keyboard
uchar pswd_cycle = 0;
#define KEYBOARD_PAUSE_KEYPRESS 3000
#define KEYBOARD_PAUSE_DELAY 60000
ushort keyboard_pause = 0;

// EEPROM
uchar eState = 0;
ushort addr = 0;
ushort endAddr = 0;
uchar usbBuffer[16];

// Calibrate oscilator and save value to eeprom
// Code is from V-USB project - osccal.c
static void calibrateOscillator(void) {
    uchar       step = 128;
    uchar       trialValue = 0, optimumValue;
    int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);
    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}
void    usbEventResetReady(void) {
    calibrateOscillator();
    if( eeprom_read_byte(0)!= OSCCAL) {
        eeprom_write_byte((uint8_t*)0, OSCCAL);
        eeprom_write_byte((uint8_t*)1, OSCCAL);
        eeprom_write_byte((uint8_t*)2, OSCCAL);
    }
    /* It is possible to change the relative error (e.g. +/-5)
     to achieve longer functionality state.
     */
}
static void hardwareDisconnect(void) {
    /* activate pull-ups except on USB lines */
    USB_CFG_IOPORT   = (uchar)~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
    /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
    USBDDR    = 0;    /* we do RESET by deactivating pullup */
    usbDeviceDisconnect();
#else
    USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif
    /* 250 ms disconnect */
    wdt_reset();
    _delay_ms(250);
}
static void hardwareConnect(void) {
#ifdef USB_CFG_PULLUP_IOPORT
    usbDeviceConnect();
#else
    USBDDR    = 0;      /*  remove USB reset condition */
#endif
}
/*------------------------------------- /COMMON --------------------------------------*/

/*------------------------------------- KEYBOARD ------------------------------------*/
static uchar reportBuffer_1[2];
static uchar idleRate_1;
PROGMEM const char usbHidReportDescriptor_1[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
uchar	usbFunctionSetup_1(uchar data[8]) {
    usbRequest_t    *rq = (void *)data;
    usbMsgPtr_1 = reportBuffer_1;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            reportBuffer_1[0] = 0;
            reportBuffer_1[1] = 0;
            return sizeof(reportBuffer_1);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr_1 = &idleRate_1;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate_1 = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}
/*------------------------------------- /KEYBOARD ------------------------------------*/

/*----------------------------------- HID SPECIFIC -----------------------------------*/
#define HID_REPORT_BUFFER_SIZE 8
static uchar reportBuffer_2[HID_REPORT_BUFFER_SIZE];
static uchar idleRate_2;
static uchar hidBytesReceied;
PROGMEM const char usbHidReportDescriptor_2[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_2] = { /* USB report descriptor */
    0x05, 0x01,                     // USAGE_PAGE (Generic Desktop)
    0x09, 0x00,                     // USAGE (Undefined)
    0xa1, 0x01,                     // COLLECTION (Application)
    0x15, 0x00,                     //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,               //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                     //   REPORT_SIZE (8-bit)
    
    0x85, 0x01,		                //   REPORT ID (1)
    0x95, HID_REPORT_BUFFER_SIZE,   //   REPORT_COUNT (HID_REPORT_BUFFER_SIZE)
    0x09, 0x00,                     //   USAGE (Undefined)
    0xb2, 0x02, 0x01,               //   FEATURE (Data,Var,Abs,Buf)
    
    0xc0                            // END_COLLECTION
};
uchar usbFunctionSetup_2(uchar data[8]) {
    usbRequest_t    *rq = (void *)data;
    usbMsgPtr_2 = reportBuffer_2;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){
            // Read from device
            
            // Blink with led
            if(!ledProcess)
                HID_BUSY
            ledProcess = 30000;
            
            // Copy Transmit buffer to report buffer
            uchar lenght = bufferTXSize(), a;
            if(lenght == 0)
                return 0;
            if(lenght>=HID_REPORT_BUFFER_SIZE)
                lenght = HID_REPORT_BUFFER_SIZE-1;
            for(a=1; a<=lenght; a++)
                reportBuffer_2[a] = bufferTXRead();
            reportBuffer_2[0] = 0;
            
            return lenght+1;  // Return length
        } else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            // Write to device
            
            // How many bytes has been already received
            hidBytesReceied = 0;
            
            return USB_NO_MSG_2;  // USB_NO_MSG_2=0xff special value for usbFunctionWrite()
        } else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr_2 = &idleRate_2;
            return 1;
        } else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate_2 = rq->wValue.bytes[1];
        }
    } else{
        /* no vendor specific requests implemented */
    }
    return 0;
}
uchar usbFunctionWrite_2( uchar *data, uchar length ) {
    
    // Ignore first byte, ID, of message
    if(hidBytesReceied == 0){
        length--;
        data++;
    }
    
    //  Write to device Receive buffer
    while(length) {
        bufferRXWrite(*data++);
		length--;
        hidBytesReceied++;
	}
    
    //  Postpone receiving next data if buffer is full
	//if( bufferRXFree()<=HID_REPORT_BUFFER_SIZE)
	//	usbDisableAllRequests();
    
    if(hidBytesReceied >= HID_REPORT_BUFFER_SIZE){
        return 1;  // - last packet
    } else {
        return 0;  // - next packet
    }
}
/*----------------------------------- /HID SPECIFIC ----------------------------------*/

/*------------------------------ INIT ONE OF USB PROFILES ----------------------------*/
static void usbInit(void){
    if(!usbVUSBconfiguration){
        usbInit_1(); // 0 - Keyboard
        processedCommand = 0;
        receivedByte = 0;
        ledProcess = 0;
        i = 0;
        KEYBOARD_OK
    } else {
        usbInit_2(); // 1 - Hid specific
        bufferReset();
        eState = 0;
        processedCommand = 0;
        commandReceivedByte = 0;
        ledProcess = 0;
        HID_OK
    }
    sei();
}
/*----------------------------- /INIT ONE OF USB PROFILES ----------------------------*/

void threewayCompare(ushort a, ushort b, ushort c, uint8_t n)
{
    while (n) {
        uint8_t ua = eeprom_read_byte((uint8_t*)a);
        uint8_t ub = eeprom_read_byte((uint8_t*)b);
        uint8_t uc = eeprom_read_byte((uint8_t*)c);
        if (ua != ub) {
            if (ua == uc) eeprom_write_byte((uint8_t*)b, uc);
            else if (ub == uc) eeprom_write_byte((uint8_t*)a, uc);
        } else if (ub != uc && ub == ua)
            eeprom_write_byte((uint8_t*)c, ua);
        ++a;
        ++b;
        ++c;
        --n;
    }
}

/*--------------------------------------- MAIN ---------------------------------------*/
int main(void)
{
    // Read oscillator calibration value from eeprom
    threewayCompare(0, 1, 2, 1);
    i = eeprom_read_byte(0);
    if (i != 0xff) {
		OSCCAL = i;
	}
    wdt_enable(WDTO_1S);

    // Initialize V-USB and HW
    //odDebugInit();
    hardwareDisconnect();
    hardwareConnect();
    usbVUSBconfiguration = HWinit();
    usbInit();

    //
    // usbEventResetReady() at first startup
    //
    
    // Main loop
    for(;;){
    
        // Poll button and led
        i = HWpoll();
        
        if(i == 255){
            // Button pressed for 5 sec - change device usb profile
            hardwareDisconnect();
            if(usbVUSBconfiguration){
                usbVUSBconfiguration = 0; // To Keyboard
            } else {
                usbVUSBconfiguration = 1; // To Hid specific
            }
            hardwareConnect();
            HWreset();
            usbInit();
        }
        wdt_reset();
        
        
        if(usbVUSBconfiguration){
            usbPoll_2(); // Poll usb
            /*------------------------ HID SPECIFIC ROUTINE ------------------------*/
            
            // EEPROM
            if(eState){
                if(eState > 3){
                    if(eState >= 6 && eeprom_is_ready()){
                        eeprom_read_block((void*)&usbBuffer, (const void*)addr, len);
                        eState = 5;
                    } else if (eState == 5) {
                        bufferTXWrite('$');
                        CRCbyte = '=';
				        bufferTXWrite('=');
                        CRCbyte += 2+len;
                        bufferTXWrite(2+len);
                        //addr -= len;
                        eState = (uchar)addr;
                        CRCbyte += eState;
                        bufferTXWrite(eState);
                        eState = (uchar)(addr>>8);
                        CRCbyte += eState;
                        bufferTXWrite(eState);
                        for(i = 0; i < len; i++){
                            CRCbyte += usbBuffer[i];
                            bufferTXWrite(usbBuffer[i]);
                        }
                        bufferTXWrite(CRCbyte); // reading done
                        eState = 4;
                    } else if (eState == 4){
                        eState = 0;
                    }
                } else {
                    if(eState == 3 && eeprom_is_ready()){
                        eeprom_write_block((const void*)&usbBuffer, (void*)addr, len);
                        eState = 2;
                    } else if (eState == 2){
                        bufferTXWrite('$');
				        bufferTXWrite('0'); // writing done
                        eState = 1;
                    } else if (eState == 1){
                        eState = 0;
                    }
                }
            }
            
	        // Hid Reports command listener
		    while(bufferRXSize() > 0 && !eState){
                receivedByte = bufferRXRead();
                if(processedCommand){
                    // There is some command in the middle of proccesing
				    
                    if(processedCommand == 1) { // '<' ////////////// Read eeprom
					    if(commandReceivedByte == 0) {
						    if(receivedByte > 17){
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('3'); // Too big for buffer
						    } else {
							    len = receivedByte; // Length
						    }
					    } else if (commandReceivedByte == 1) {
						    addr = receivedByte; // ADDRL
					    } else if (commandReceivedByte == 2) {
                            addr += receivedByte<<8; // ADDRH
						    /*if (addr < 5) {
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('4'); // First bytes in eeprom protected (OSCCAL)
						    } else*/ if (addr > ((ushort)512-len)){
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('2'); // Overflow (512)
						    }
					    } else if (commandReceivedByte == 3) {
                            processedCommand = 0;
                            if(receivedByte == CRCbyte){  // CRC
                                if(!len){
                                    bufferTXWrite('$');
							        bufferTXWrite('0'); // nothing to read
			                    } else {
                                    eState = 6; // read
                                }
						    } else {
                                bufferTXWrite('$');
							    bufferTXWrite('1'); // wrong crc
						    }
					    }
					    CRCbyte += receivedByte;
					    commandReceivedByte++;
                        
				    } else if (processedCommand == 2) { // # ////////////// Write to eeprom
					    if(commandReceivedByte == 0) {
						    if(receivedByte > 16 || receivedByte < 3){  // length
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('3'); // Too long or too short
						    } else {
							    len = receivedByte + 1; // Length
						    }
					    } else if (commandReceivedByte == 1) {
						    addr = receivedByte; // ADDRL
					    } else if (commandReceivedByte == 2) {
                            addr += receivedByte<<8; // ADDRH
                            /*if (addr < 5){
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('4'); // First bytes in eeprom protected (OSCCAL)
						    } else*/ if (addr > ((ushort)512-len)){
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('2'); // Overflow (512)
						    }
					    } else if (commandReceivedByte < len) {
                            usbBuffer[commandReceivedByte-3] = receivedByte; // Store data to buffer
					    } else if (commandReceivedByte == len) {
                            processedCommand = 0;
                            if(receivedByte == CRCbyte){  // CRC
                                len -= 3;
                                if(!len) {
				                    bufferTXWrite('$');
				                    bufferTXWrite('0');  // nothing to write
			                    } else {
                                    eState = 3; // write
                                }
						    } else {
                                bufferTXWrite('$');
							    bufferTXWrite('1'); // wrong crc
						    }
					    }
                        CRCbyte += receivedByte; // counting CRC
					    commandReceivedByte++; // next instruction
                        
				    } else if (processedCommand == 3) { // > ////////////// Return parameters
                        if(commandReceivedByte == 0) {
						    if(receivedByte != 1){  // length
							    processedCommand = 0;
							    bufferTXWrite('$');
							    bufferTXWrite('2'); // Invalid length (not supported)
						    } else {
							    len = receivedByte; // Length
						    }
					    } else {
						    processedCommand = 0;
                            if(receivedByte == CRCbyte){  // CRC
                                commandReceivedByte = OSCCAL;
					            bufferTXWrite('$');
					            bufferTXWrite('@');
					            bufferTXWrite(1);
					            bufferTXWrite(commandReceivedByte);
					            bufferTXWrite((('@'+1+commandReceivedByte)&0xFF));
						    } else {
                                bufferTXWrite('$');
							    bufferTXWrite('1'); // wrong crc
						    }
					    }
                        CRCbyte += receivedByte; // counting CRC
					    commandReceivedByte++; // next received byte
                    } else if (processedCommand == 4) { // ^ ////////////// Calibrate
                        if(commandReceivedByte == 0) {
                            if(receivedByte != 0x2b){  // check byte 1
                                processedCommand = 0;
                                bufferTXWrite('$');
                                bufferTXWrite('2');
                            }
                        } else if(commandReceivedByte == 1) {
                            if(receivedByte != 0x95){  // check byte 2
                                processedCommand = 0;
                                bufferTXWrite('$');
                                bufferTXWrite('3');
                            }
                        } else {
                            processedCommand = 0;
                            if(receivedByte == CRCbyte){  // CRC
                                usbEventResetReady();
                                //calibrateOscillator();
                            } else {
                                bufferTXWrite('$');
                                bufferTXWrite('1'); // wrong crc
                            }
                        }
                        CRCbyte += receivedByte; // counting CRC
                        commandReceivedByte++; // next received byte
                    } else {
					    processedCommand = 0;
				    }
				    if(!processedCommand){
					    if(receivedByte == '$') {
						    commandReceivedByte = 1;
					    } else {
						    commandReceivedByte = 0;
					    }
				    }
                    
			    } else {
                    // No command in process
				    if(receivedByte == '$') {
					    commandReceivedByte = 1; // Starting new message
				    } else if (receivedByte == '<' && commandReceivedByte == 1) {
					    CRCbyte = '<'; // Read eeprom
					    commandReceivedByte = 0;
					    processedCommand = 1;
				    } else if (receivedByte == '#' && commandReceivedByte == 1) {
					    CRCbyte = '#'; // Write eeprom
					    commandReceivedByte = 0;
					    processedCommand = 2;
				    } else if (receivedByte == '>' && commandReceivedByte == 1) {
                        CRCbyte = '>'; // Return parameters (OSCCAL)
                        commandReceivedByte = 0;
                        processedCommand = 3;
                    } else if (receivedByte == '^' && commandReceivedByte == 1) {
                        CRCbyte = '^'; // calibrate
                        commandReceivedByte = 0;
                        processedCommand = 4;
                    } else if (receivedByte == '?' && commandReceivedByte == 1) {
                        // Return device ID
					    bufferTXWrite('$');
					    bufferTXWrite('!');
					    bufferTXWrite(7);
					    bufferTXWrite('P');
					    bufferTXWrite('s');
					    bufferTXWrite('w');
					    bufferTXWrite('d');
					    bufferTXWrite('K');
					    bufferTXWrite('e');
					    bufferTXWrite('y');
					    bufferTXWrite((('!'+7+'P'+'s'+'w'+'d'+'K'+'e'+'y')&0xFF));
					    commandReceivedByte = 0;
				    } else {
					    commandReceivedByte = 0;
				    }
			    }
		    }
            
            // Stop with led flashing after some time
            if(ledProcess){
                ledProcess--;
                if(!ledProcess)
                    HID_OK
            }
            
            // Enable usb requests if buffer is not full
            //if( usbAllRequestsAreDisabled() && bufferRXFree()>HID_REPORT_BUFFER_SIZE)
            //    usbEnableAllRequests();

            /*----------------------- /HID SPECIFIC ROUTINE ------------------------*/
        } else {
            usbPoll_1(); // Poll usb
            /*------------------------------ KEYBOARD ------------------------------*/
        
            // Emulate virtual keys presses
            if (keyboard_pause) {
                --keyboard_pause;
            } else if(processedCommand && !receivedByte) {
                ledProcess = 30000;
                if(processedCommand == 1){
                    // Emulate virtual key press
                    keyboard_pause = KEYBOARD_PAUSE_KEYPRESS;
                    processedCommand = 2;
                    receivedByte = 1;
                    
                    reportBuffer_1[0] = 0; // no modifiers
                    if ((addr>>8) == 0xff) {
                        reportBuffer_1[1] = usbBuffer[(addr++)&0xff]; // key
                    } else {
                        reportBuffer_1[1] = 0; // no key
                        eState = 0;
                        while (addr < endAddr) {
                            size = eeprom_read_byte((uint8_t*)(addr++));
                            if (eState) {
                                eState = 0;
                                if (size == 0x01) {
                                    keyboard_pause = KEYBOARD_PAUSE_DELAY;
                                    size = 0;
                                }
                                reportBuffer_1[1] = size;
                                break;
                            } else if (size == 0xff) {
                                eState = 1;
                            } else if (size&0x80) {
                                reportBuffer_1[0] |= size&0x7f;
                            } else {
                                if (size&0x40) reportBuffer_1[0] |= 2;
                                if ((size&0x3f) == 0x01) {
                                    keyboard_pause = KEYBOARD_PAUSE_DELAY;
                                    size = 0;
                                }
                                reportBuffer_1[1] = size&0x3f;
                                break;
                            }
                        }
                    }
                    if (!reportBuffer_1[0] && !reportBuffer_1[1] &&
                        keyboard_pause != KEYBOARD_PAUSE_DELAY) {
                        processedCommand = 0; // No other letter - stop
                        receivedByte = 0;
                    }
                } else {
                    // Emulate virtual key release
                    reportBuffer_1[0] = 0; // no modifiers
                    reportBuffer_1[1] = 0; // no key
                    // Go to next letter
                    if(addr < endAddr){
                        processedCommand = 1;
                    } else {
                        processedCommand = 0; // No other letter - stop
                    }
                    keyboard_pause = KEYBOARD_PAUSE_KEYPRESS;
                    receivedByte = 1;
                }
            }

            // Stop with led flashing after some time
            if(ledProcess){
                ledProcess--;
                if(!ledProcess)
                    KEYBOARD_OK
            }

            // Button pressed - write according password
            if(i && !processedCommand) {
                pswd_cycle = (pswd_cycle+1)%3; // 0, 1, 2
                // Read items number
                addr = (ushort)pswd_cycle*(ushort)170+10;
                size = CRCbyte = eeprom_read_byte((uint8_t*)(addr));
                if (CRCbyte != (uchar)(~eeprom_read_byte((uint8_t*)(addr+1)))) {
                    // error
                    // repair first 2 bytes
                    threewayCompare(10, 180, 350, 2);
                    // test if working
                    CRCbyte = eeprom_read_byte((uint8_t*)(addr));
                    if (CRCbyte == (uchar)~eeprom_read_byte((uint8_t*)(addr+1))) {
                        // repaired
                        size = CRCbyte;
                    } else {
                        // not repaired
                        size = 0;
                    }
                }
                if (i <= (size&0x7f)) {
                    len = 0; // length of password i
                    addr += 2;
                    endAddr = 0;
                    for (eState = 0; eState < i; ++eState) {
                        endAddr += len;
                        len = eeprom_read_byte((uint8_t*)(addr+eState));
                        CRCbyte += len;
                    }
                    for (;eState < (size&0x7f); ++eState) {
                        CRCbyte += (uchar)eeprom_read_byte((uint8_t*)(addr+eState));
                    }
                    if (CRCbyte != (uchar)eeprom_read_byte((uint8_t*)(addr+eState))) {
                        // crc error
                        // try repair first size+3 bytes
                        threewayCompare(10, 180, 350, 3+(size&0x7f));
                        // test if working
                        len = 0; // length of password i
                        CRCbyte = size;
                        endAddr = 0;
                        for (eState = 0; eState < i; ++eState) {
                            endAddr += len;
                            len = eeprom_read_byte((uint8_t*)(addr+eState));
                            CRCbyte += len;
                        }
                        for (;eState < (size&0x7f); ++eState) {
                            CRCbyte += (uchar)eeprom_read_byte((uint8_t*)(addr+eState));
                        }
                        if (CRCbyte != (uchar)eeprom_read_byte((uint8_t*)(addr+eState))) {
                            // not repaired
                            size = 0;
                        }
                    }
                }
                size &= 0x7f;
                if (i <= size) {
                    // check crc
                    addr += (size+1)+endAddr; // start of the password
                    CRCbyte = 0;
                    --len;
                    for (eState = 0;eState < len; ++eState) {
                        CRCbyte += eeprom_read_byte((uint8_t*)(addr+eState));
                    }
                    if (CRCbyte != (uchar)eeprom_read_byte((uint8_t*)(addr+eState))) {
                        // crc error
                        // repair from endAddr len+1 bytes
                        threewayCompare(13+endAddr+size,
                                        183+endAddr+size,
                                        353+endAddr+size, len+1);
                        // check again
                        CRCbyte = 0;
                        for (eState = 0;eState < len; ++eState) {
                            CRCbyte += eeprom_read_byte((uint8_t*)(addr+eState));
                        }
                        if (CRCbyte != (uchar)eeprom_read_byte((uint8_t*)(addr+eState))) {
                            // not repaired
                            size = 0;
                        }
                    }
                    endAddr = addr+len;
                    keyboard_pause = 0;
                    processedCommand = 1;
                }
                if (!size) {
                    // memory error
                    usbBuffer[0] = 'm'-'a'+4;
                    usbBuffer[1] = 'e'-'a'+4;
                    usbBuffer[2] = 'm'-'a'+4;
                    usbBuffer[3] = 'o'-'a'+4;
                    usbBuffer[4] = 'r'-'a'+4;
                    usbBuffer[5] = 'y'-'a'+4;
                    usbBuffer[6] = 0x2c;
                    usbBuffer[7] = 'e'-'a'+4;
                    usbBuffer[8] = 'r'-'a'+4;
                    usbBuffer[9] = 'r'-'a'+4;
                    usbBuffer[10]= 'o'-'a'+4;
                    usbBuffer[11]= 'r'-'a'+4;
                    usbBuffer[12] = 0x2c;
                    usbBuffer[13] = 0x2c;
                    addr = 0xff00;
                    endAddr = addr+14;
                    processedCommand = 1;
                }
                KEYBOARD_BUSY
                ledProcess = 30000;
            }

		    // Send usb key press/release if any
		    if(usbInterruptIsReady_1() && receivedByte) { /* we can send another key */
            	usbSetInterrupt_1(reportBuffer_1, sizeof(reportBuffer_1));
                receivedByte = 0;
            }

            /*----------------------------- /KEYBOARD ------------------------------*/
        }
	}
    return 0;
}

