/*
 * Name: usbdrv.c
 * Project: V-USB, virtual USB port for Atmel's(r) AVR(r) microcontrollers
 * Author: Christian Starkjohann
 * Creation Date: 2004-12-29
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 */

/*
 * Name: usbdrv_1.c
 * Edited By: Jiri Stepanovsky - http://jiristepanovsky.cz
 * For Project: PassKeychain
 * Date: 2014-12-05
 * License: GNU GPL v2, GNU GPL v3 or proprietary
 */

#include "usbdrv_1.h"
#include "oddebug.h"

/*
General Description:
This module implements the C-part of the USB driver. See usbdrv.h for a
documentation of the entire driver.
*/


/* USB status registers / not shared with asm code */
usbMsgPtr_t_1         usbMsgPtr_1;      /* data to transmit next -- ROM or RAM address */
static usbMsgLen_t_1  usbMsgLen_1 = USB_NO_MSG_1; /* remaining number of bytes */
static uchar        usbMsgFlags_1;    /* flag values see below */

/* -------------------------- String Descriptors --------------------------- */

#if USB_CFG_DESCR_PROPS_STRINGS_1 == 0

#if USB_CFG_DESCR_PROPS_STRING_0_1 == 0
#undef USB_CFG_DESCR_PROPS_STRING_0_1
#define USB_CFG_DESCR_PROPS_STRING_0_1    sizeof(usbDescriptorString0_1)
PROGMEM const char usbDescriptorString0_1[] = { /* language descriptor */
    4,          /* sizeof(usbDescriptorString0_1): length of descriptor in bytes */
    3,          /* descriptor type */
    0x09, 0x04, /* language index (0x0409 = US-English) */
};
#endif

#if USB_CFG_DESCR_PROPS_STRING_VENDOR_1 == 0 && USB_CFG_VENDOR_NAME_LEN_1
#undef USB_CFG_DESCR_PROPS_STRING_VENDOR_1
#define USB_CFG_DESCR_PROPS_STRING_VENDOR_1   sizeof(usbDescriptorStringVendor_1)
PROGMEM const int  usbDescriptorStringVendor_1[] = {
    USB_STRING_DESCRIPTOR_HEADER(USB_CFG_VENDOR_NAME_LEN_1),
    USB_CFG_VENDOR_NAME_1
};
#endif

#if USB_CFG_DESCR_PROPS_STRING_PRODUCT_1 == 0 && USB_CFG_DEVICE_NAME_LEN_1
#undef USB_CFG_DESCR_PROPS_STRING_PRODUCT_1
#define USB_CFG_DESCR_PROPS_STRING_PRODUCT_1   sizeof(usbDescriptorStringDevice_1)
PROGMEM const int  usbDescriptorStringDevice_1[] = {
    USB_STRING_DESCRIPTOR_HEADER(USB_CFG_DEVICE_NAME_LEN_1),
    USB_CFG_DEVICE_NAME_1
};
#endif

#if USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1 == 0 && USB_CFG_SERIAL_NUMBER_LEN_1
#undef USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1
#define USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1    sizeof(usbDescriptorStringSerialNumber_1)
PROGMEM const int usbDescriptorStringSerialNumber_1[] = {
    USB_STRING_DESCRIPTOR_HEADER(USB_CFG_SERIAL_NUMBER_LEN_1),
    USB_CFG_SERIAL_NUMBER_1
};
#endif

#endif  /* USB_CFG_DESCR_PROPS_STRINGS == 0 */

/* --------------------------- Device Descriptor --------------------------- */

#if USB_CFG_DESCR_PROPS_DEVICE_1 == 0
#undef USB_CFG_DESCR_PROPS_DEVICE_1
#define USB_CFG_DESCR_PROPS_DEVICE_1  sizeof(usbDescriptorDevice_1)
PROGMEM const char usbDescriptorDevice_1[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x10, 0x01,             /* USB version supported */
    USB_CFG_DEVICE_CLASS_1,
    USB_CFG_DEVICE_SUBCLASS_1,
    0,                      /* protocol */
    8,                      /* max packet size */
    /* the following two casts affect the first byte of the constant only, but
     * that's sufficient to avoid a warning with the default values.
     */
    (char)USB_CFG_VENDOR_ID_1,/* 2 bytes */
    (char)USB_CFG_DEVICE_ID_1,/* 2 bytes */
    USB_CFG_DEVICE_VERSION_1, /* 2 bytes */
    USB_CFG_DESCR_PROPS_STRING_VENDOR_1 != 0 ? 1 : 0,         /* manufacturer string index */
    USB_CFG_DESCR_PROPS_STRING_PRODUCT_1 != 0 ? 2 : 0,        /* product string index */
    USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1 != 0 ? 3 : 0,  /* serial number string index */
    1,          /* number of configurations */
};
#endif

/* ----------------------- Configuration Descriptor ------------------------ */

#if USB_CFG_DESCR_PROPS_HID_REPORT_1 != 0 && USB_CFG_DESCR_PROPS_HID_1 == 0
#undef USB_CFG_DESCR_PROPS_HID_1
#define USB_CFG_DESCR_PROPS_HID_1     9   /* length of HID descriptor in config descriptor below */
#endif

#if USB_CFG_DESCR_PROPS_CONFIGURATION_1 == 0
#undef USB_CFG_DESCR_PROPS_CONFIGURATION_1
#define USB_CFG_DESCR_PROPS_CONFIGURATION_1   sizeof(usbDescriptorConfiguration_1)
PROGMEM const char usbDescriptorConfiguration_1[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescriptorConfiguration_1): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    18 + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT_1 + 7 * USB_CFG_HAVE_INTRIN_ENDPOINT3_1 +
                (USB_CFG_DESCR_PROPS_HID_1 & 0xff), 0,
                /* total length of data returned (including inlined descriptors) */
    1,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER,       /* attributes */
#else
    (1 << 7),                           /* attributes */
#endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */
/* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT_1 + USB_CFG_HAVE_INTRIN_ENDPOINT3_1, /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS_1,
    USB_CFG_INTERFACE_SUBCLASS_1,
    USB_CFG_INTERFACE_PROTOCOL_1,
    0,          /* string index for interface */
#if (USB_CFG_DESCR_PROPS_HID_1 & 0xff)    /* HID descriptor */
    9,          /* sizeof(usbDescrHID_1): length of descriptor in bytes */
    USBDESCR_HID,   /* descriptor type: HID */
    0x01, 0x01, /* BCD representation of HID version */
    0x00,       /* target country code */
    0x01,       /* number of HID Report (or other HID class) Descriptor infos to follow */
    0x22,       /* descriptor type: report */
    USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1, 0,  /* total length of report descriptor */
#endif
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1    /* endpoint descriptor for endpoint 1 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)0x81, /* IN endpoint number 1 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL_1, /* in ms */
#endif
#if USB_CFG_HAVE_INTRIN_ENDPOINT3_1   /* endpoint descriptor for endpoint 3 */
    7,          /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    (char)(0x80 | USB_CFG_EP3_NUMBER), /* IN endpoint number 3 */
    0x03,       /* attrib: Interrupt endpoint */
    8, 0,       /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL_1, /* in ms */
#endif
};
#endif

/* ------------------------------------------------------------------------- */

static inline void  usbResetDataToggling_1(void)
{
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1 && !USB_CFG_SUPPRESS_INTR_CODE_1
    USB_SET_DATATOKEN1(USB_INITIAL_DATATOKEN);  /* reset data toggling for interrupt endpoint */
#   if USB_CFG_HAVE_INTRIN_ENDPOINT3_1
    USB_SET_DATATOKEN3(USB_INITIAL_DATATOKEN);  /* reset data toggling for interrupt endpoint */
#   endif
#endif
}

static inline void  usbResetStall_1(void)
{
#if USB_CFG_IMPLEMENT_HALT_1 && USB_CFG_HAVE_INTRIN_ENDPOINT_1
        usbTxLen1 = USBPID_NAK;
#if USB_CFG_HAVE_INTRIN_ENDPOINT3_1
        usbTxLen3 = USBPID_NAK;
#endif
#endif
}

/* ------------------------------------------------------------------------- */

#if !USB_CFG_SUPPRESS_INTR_CODE_1
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1
static void usbGenericSetInterrupt_1(uchar *data, uchar len, usbTxStatus_t *txStatus)
{
uchar   *p;
char    i;

#if USB_CFG_IMPLEMENT_HALT_1
    if(usbTxLen1 == USBPID_STALL)
        return;
#endif
    if(txStatus->len & 0x10){   /* packet buffer was empty */
        txStatus->buffer[0] ^= USBPID_DATA0 ^ USBPID_DATA1; /* toggle token */
    }else{
        txStatus->len = USBPID_NAK; /* avoid sending outdated (overwritten) interrupt data */
    }
    p = txStatus->buffer + 1;
    i = len;
    do{                         /* if len == 0, we still copy 1 byte, but that's no problem */
        *p++ = *data++;
    }while(--i > 0);            /* loop control at the end is 2 bytes shorter than at beginning */
    usbCrc16Append(&txStatus->buffer[1], len);
    txStatus->len = len + 4;    /* len must be given including sync byte */
    DBG2(0x21 + (((int)txStatus >> 3) & 3), txStatus->buffer, len + 3);
}

USB_PUBLIC void usbSetInterrupt_1(uchar *data, uchar len)
{
    usbGenericSetInterrupt_1(data, len, &usbTxStatus1);
}
#endif

#if USB_CFG_HAVE_INTRIN_ENDPOINT3_1
USB_PUBLIC void usbSetInterrupt3_1(uchar *data, uchar len)
{
    usbGenericSetInterrupt_1(data, len, &usbTxStatus3);
}
#endif
#endif /* USB_CFG_SUPPRESS_INTR_CODE */

/* ------------------------------------------------------------------------- */

#ifndef USB_RX_USER_HOOK_1
#define USB_RX_USER_HOOK_1(data, len)
#endif
#ifndef USB_SET_ADDRESS_HOOK_1
#define USB_SET_ADDRESS_HOOK_1()
#endif

/* We use if() instead of #if in the macro below because #if can't be used
 * in macros and the compiler optimizes constant conditions anyway.
 * This may cause problems with undefined symbols if compiled without
 * optimizing!
 */
#define GET_DESCRIPTOR_1(cfgProp, staticName)         \
    if(cfgProp){                                    \
        if((cfgProp) & USB_PROP_IS_RAM)             \
            flags = 0;                              \
        if((cfgProp) & USB_PROP_IS_DYNAMIC){        \
            len = usbFunctionDescriptor_1(rq);        \
        }else{                                      \
            len = USB_PROP_LENGTH(cfgProp);         \
            usbMsgPtr_1 = (usbMsgPtr_t_1)(staticName);  \
        }                                           \
    }

/* usbDriverDescriptor_1() is similar to usbFunctionDescriptor_1(), but used
 * internally for all types of descriptors.
 */
static inline usbMsgLen_t_1 usbDriverDescriptor_1(usbRequest_t *rq)
{
usbMsgLen_t_1 len = 0;
uchar       flags = USB_FLG_MSGPTR_IS_ROM;

    SWITCH_START(rq->wValue.bytes[1])
    SWITCH_CASE(USBDESCR_DEVICE)    /* 1 */
        GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_DEVICE_1, usbDescriptorDevice_1)
    SWITCH_CASE(USBDESCR_CONFIG)    /* 2 */
        GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_CONFIGURATION_1, usbDescriptorConfiguration_1)
    SWITCH_CASE(USBDESCR_STRING)    /* 3 */
#if USB_CFG_DESCR_PROPS_STRINGS_1 & USB_PROP_IS_DYNAMIC
        if(USB_CFG_DESCR_PROPS_STRINGS_1 & USB_PROP_IS_RAM)
            flags = 0;
        len = usbFunctionDescriptor_1(rq);
#else   /* USB_CFG_DESCR_PROPS_STRINGS_1 & USB_PROP_IS_DYNAMIC */
        SWITCH_START(rq->wValue.bytes[0])
        SWITCH_CASE(0)
            GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_STRING_0_1, usbDescriptorString0_1)
        SWITCH_CASE(1)
            GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_STRING_VENDOR_1, usbDescriptorStringVendor_1)
        SWITCH_CASE(2)
            GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_STRING_PRODUCT_1, usbDescriptorStringDevice_1)
        SWITCH_CASE(3)
            GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1, usbDescriptorStringSerialNumber_1)
        SWITCH_DEFAULT
            if(USB_CFG_DESCR_PROPS_UNKNOWN_1 & USB_PROP_IS_DYNAMIC){
                len = usbFunctionDescriptor_1(rq);
            }
        SWITCH_END
#endif  /* USB_CFG_DESCR_PROPS_STRINGS & USB_PROP_IS_DYNAMIC */
#if USB_CFG_DESCR_PROPS_HID_REPORT_1  /* only support HID descriptors if enabled */
    SWITCH_CASE(USBDESCR_HID)       /* 0x21 */
        GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_HID_1, usbDescriptorConfiguration_1 + 18)
    SWITCH_CASE(USBDESCR_HID_REPORT)/* 0x22 */
        GET_DESCRIPTOR_1(USB_CFG_DESCR_PROPS_HID_REPORT_1, usbDescriptorHidReport_1)
#endif
    SWITCH_DEFAULT
        if(USB_CFG_DESCR_PROPS_UNKNOWN_1 & USB_PROP_IS_DYNAMIC){
            len = usbFunctionDescriptor_1(rq);
        }
    SWITCH_END
    usbMsgFlags_1 = flags;
    return len;
}

/* ------------------------------------------------------------------------- */

/* usbDriverSetup_1() is similar to usbFunctionSetup_1(), but it's used for
 * standard requests instead of class and custom requests.
 */
static inline usbMsgLen_t_1 usbDriverSetup_1(usbRequest_t *rq)
{
usbMsgLen_t_1 len = 0;
uchar   *dataPtr = usbTxBuf + 9;    /* there are 2 bytes free space at the end of the buffer */
uchar   value = rq->wValue.bytes[0];
#if USB_CFG_IMPLEMENT_HALT_1
uchar   index = rq->wIndex.bytes[0];
#endif

    dataPtr[0] = 0; /* default reply common to USBRQ_GET_STATUS and USBRQ_GET_INTERFACE */
    SWITCH_START(rq->bRequest)
    SWITCH_CASE(USBRQ_GET_STATUS)           /* 0 */
        uchar recipient = rq->bmRequestType & USBRQ_RCPT_MASK;  /* assign arith ops to variables to enforce byte size */
        if(USB_CFG_IS_SELF_POWERED && recipient == USBRQ_RCPT_DEVICE)
            dataPtr[0] =  USB_CFG_IS_SELF_POWERED;
#if USB_CFG_IMPLEMENT_HALT_1
        if(recipient == USBRQ_RCPT_ENDPOINT && index == 0x81)   /* request status for endpoint 1 */
            dataPtr[0] = usbTxLen1 == USBPID_STALL;
#endif
        dataPtr[1] = 0;
        len = 2;
#if USB_CFG_IMPLEMENT_HALT_1
    SWITCH_CASE2(USBRQ_CLEAR_FEATURE, USBRQ_SET_FEATURE)    /* 1, 3 */
        if(value == 0 && index == 0x81){    /* feature 0 == HALT for endpoint == 1 */
            usbTxLen1 = rq->bRequest == USBRQ_CLEAR_FEATURE ? USBPID_NAK : USBPID_STALL;
            usbResetDataToggling_1();
        }
#endif
    SWITCH_CASE(USBRQ_SET_ADDRESS)          /* 5 */
        usbNewDeviceAddr = value;
        USB_SET_ADDRESS_HOOK_1();
    SWITCH_CASE(USBRQ_GET_DESCRIPTOR)       /* 6 */
        len = usbDriverDescriptor_1(rq);
        goto skipMsgPtrAssignment;
    SWITCH_CASE(USBRQ_GET_CONFIGURATION)    /* 8 */
        dataPtr = &usbConfiguration;  /* send current configuration value */
        len = 1;
    SWITCH_CASE(USBRQ_SET_CONFIGURATION)    /* 9 */
        usbConfiguration = value;
        usbResetStall_1();
    SWITCH_CASE(USBRQ_GET_INTERFACE)        /* 10 */
        len = 1;
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1 && !USB_CFG_SUPPRESS_INTR_CODE_1
    SWITCH_CASE(USBRQ_SET_INTERFACE)        /* 11 */
        usbResetDataToggling_1();
        usbResetStall_1();
#endif
    SWITCH_DEFAULT                          /* 7=SET_DESCRIPTOR, 12=SYNC_FRAME */
        /* Should we add an optional hook here? */
    SWITCH_END
    usbMsgPtr_1 = (usbMsgPtr_t_1)dataPtr;
skipMsgPtrAssignment:
    return len;
}

/* ------------------------------------------------------------------------- */

/* usbProcessRx_1() is called for every message received by the interrupt
 * routine. It distinguishes between SETUP and DATA packets and processes
 * them accordingly.
 */
static inline void usbProcessRx_1(uchar *data, uchar len)
{
usbRequest_t    *rq = (void *)data;

/* usbRxToken can be:
 * 0x2d 00101101 (USBPID_SETUP for setup data)
 * 0xe1 11100001 (USBPID_OUT: data phase of setup transfer)
 * 0...0x0f for OUT on endpoint X
 */
    DBG2(0x10 + (usbRxToken & 0xf), data, len + 2); /* SETUP=1d, SETUP-DATA=11, OUTx=1x */
    USB_RX_USER_HOOK_1(data, len)
#if USB_CFG_IMPLEMENT_FN_WRITEOUT_1
    if(usbRxToken < 0x10){  /* OUT to endpoint != 0: endpoint number in usbRxToken */
        usbFunctionWriteOut_1(data, len);
        return;
    }
#endif
    if(usbRxToken == (uchar)USBPID_SETUP){
        if(len != 8)    /* Setup size must be always 8 bytes. Ignore otherwise. */
            return;
        usbMsgLen_t_1 replyLen;
        usbTxBuf[0] = USBPID_DATA0;         /* initialize data toggling */
        usbTxLen = USBPID_NAK;              /* abort pending transmit */
        usbMsgFlags_1 = 0;
        uchar type = rq->bmRequestType & USBRQ_TYPE_MASK;
        if(type != USBRQ_TYPE_STANDARD){    /* standard requests are handled by driver */
            replyLen = usbFunctionSetup_1(data);
        }else{
            replyLen = usbDriverSetup_1(rq);
        }
#if USB_CFG_IMPLEMENT_FN_READ_1 || USB_CFG_IMPLEMENT_FN_WRITE_1
        if(replyLen == USB_NO_MSG_1){         /* use user-supplied read/write function */
            /* do some conditioning on replyLen, but on IN transfers only */
            if((rq->bmRequestType & USBRQ_DIR_MASK) != USBRQ_DIR_HOST_TO_DEVICE){
                if(sizeof(replyLen) < sizeof(rq->wLength.word)){ /* help compiler with optimizing */
                    replyLen = rq->wLength.bytes[0];
                }else{
                    replyLen = rq->wLength.word;
                }
            }
            usbMsgFlags_1 = USB_FLG_USE_USER_RW;
        }else   /* The 'else' prevents that we limit a replyLen of USB_NO_MSG to the maximum transfer len. */
#endif
        if(sizeof(replyLen) < sizeof(rq->wLength.word)){ /* help compiler with optimizing */
            if(!rq->wLength.bytes[1] && replyLen > rq->wLength.bytes[0])    /* limit length to max */
                replyLen = rq->wLength.bytes[0];
        }else{
            if(replyLen > rq->wLength.word)     /* limit length to max */
                replyLen = rq->wLength.word;
        }
        usbMsgLen_1 = replyLen;
    }else{  /* usbRxToken must be USBPID_OUT, which means data phase of setup (control-out) */
#if USB_CFG_IMPLEMENT_FN_WRITE_1
        if(usbMsgFlags_1 & USB_FLG_USE_USER_RW){
            uchar rval = usbFunctionWrite_1(data, len);
            if(rval == 0xff){   /* an error occurred */
                usbTxLen = USBPID_STALL;
            }else if(rval != 0){    /* This was the final package */
                usbMsgLen_1 = 0;  /* answer with a zero-sized data packet */
            }
        }
#endif
    }
}

/* ------------------------------------------------------------------------- */

/* This function is similar to usbFunctionRead_1(), but it's also called for
 * data handled automatically by the driver (e.g. descriptor reads).
 */
static uchar usbDeviceRead_1(uchar *data, uchar len)
{
    if(len > 0){    /* don't bother app with 0 sized reads */
#if USB_CFG_IMPLEMENT_FN_READ_1
        if(usbMsgFlags_1 & USB_FLG_USE_USER_RW){
            len = usbFunctionRead_1(data, len);
        }else
#endif
        {
            uchar i = len;
            usbMsgPtr_t_1 r = usbMsgPtr_1;
            if(usbMsgFlags_1 & USB_FLG_MSGPTR_IS_ROM){    /* ROM data */
                do{
                    uchar c = USB_READ_FLASH(r);    /* assign to char size variable to enforce byte ops */
                    *data++ = c;
                    r++;
                }while(--i);
            }else{  /* RAM data */
                do{
                    *data++ = *((uchar *)r);
                    r++;
                }while(--i);
            }
            usbMsgPtr_1 = r;
        }
    }
    return len;
}

/* ------------------------------------------------------------------------- */

/* usbBuildTxBlock_1() is called when we have data to transmit and the
 * interrupt routine's transmit buffer is empty.
 */
static inline void usbBuildTxBlock_1(void)
{
usbMsgLen_t_1 wantLen;
uchar       len;

    wantLen = usbMsgLen_1;
    if(wantLen > 8)
        wantLen = 8;
    usbMsgLen_1 -= wantLen;
    usbTxBuf[0] ^= USBPID_DATA0 ^ USBPID_DATA1; /* DATA toggling */
    len = usbDeviceRead_1(usbTxBuf + 1, wantLen);
    if(len <= 8){           /* valid data packet */
        usbCrc16Append(&usbTxBuf[1], len);
        len += 4;           /* length including sync byte */
        if(len < 12)        /* a partial package identifies end of message */
            usbMsgLen_1 = USB_NO_MSG_1;
    }else{
        len = USBPID_STALL;   /* stall the endpoint */
        usbMsgLen_1 = USB_NO_MSG_1;
    }
    usbTxLen = len;
    DBG2(0x20, usbTxBuf, len-1);
}


/* ------------------------------------------------------------------------- */

USB_PUBLIC void usbPoll_1(void)
{
schar   len;
uchar   i;

    len = usbRxLen - 3;
    if(len >= 0){
/* We could check CRC16 here -- but ACK has already been sent anyway. If you
 * need data integrity checks with this driver, check the CRC in your app
 * code and report errors back to the host. Since the ACK was already sent,
 * retries must be handled on application level.
 * unsigned crc = usbCrc16(buffer + 1, usbRxLen - 3);
 */
        usbProcessRx_1(usbRxBuf + USB_BUFSIZE + 1 - usbInputBufOffset, len);
#if USB_CFG_HAVE_FLOWCONTROL_1
        if(usbRxLen > 0)    /* only mark as available if not inactivated */
            usbRxLen = 0;
#else
        usbRxLen = 0;       /* mark rx buffer as available */
#endif
    }
    if(usbTxLen & 0x10){    /* transmit system idle */
        if(usbMsgLen_1 != USB_NO_MSG_1){    /* transmit data pending? */
            usbBuildTxBlock_1();
        }
    }
    for(i = 20; i > 0; i--){
        uchar usbLineStatus = USBIN & USBMASK;
        if(usbLineStatus != 0)  /* SE0 has ended */
            goto isNotReset;
    }
    /* RESET condition, called multiple times during reset */
    usbNewDeviceAddr = 0;
    usbDeviceAddr = 0;
    usbResetStall_1();
    DBG1(0xff, 0, 0);
isNotReset:
    usbHandleResetHook(i);
}

/* ------------------------------------------------------------------------- */

USB_PUBLIC void usbInit_1(void)
{
    usbMsgLen_1 = USB_NO_MSG_1;
#if USB_INTR_CFG_SET != 0
    USB_INTR_CFG |= USB_INTR_CFG_SET;
#endif
#if USB_INTR_CFG_CLR != 0
    USB_INTR_CFG &= ~(USB_INTR_CFG_CLR);
#endif
    USB_INTR_ENABLE |= (1 << USB_INTR_ENABLE_BIT);
    usbResetDataToggling_1();
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1 && !USB_CFG_SUPPRESS_INTR_CODE_1
    usbTxLen1 = USBPID_NAK;
#if USB_CFG_HAVE_INTRIN_ENDPOINT3_1
    usbTxLen3 = USBPID_NAK;
#endif
#endif
}

/* ------------------------------------------------------------------------- */
