/*
 * Name: usbdrv.h
 * Project: V-USB, virtual USB port for Atmel's(r) AVR(r) microcontrollers
 * Author: Christian Starkjohann
 * Creation Date: 2004-12-29
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 */

/*
 * Name: usbdrv_1.h
 * Edited By: Jiri Stepanovsky - http://jiristepanovsky.cz
 * For Project: PassKeychain
 * Date: 2014-12-05
 * License: GNU GPL v2, GNU GPL v3 or proprietary
 */

#ifndef __usbdrv_1_h_included__
#define __usbdrv_1_h_included__

#include "usbdrvCommon.h"
#include "usbconfig_1.h"
#include "usbportability.h"


#ifndef __ASSEMBLER__
#ifndef uchar
#define uchar   unsigned char
#endif
#ifndef schar
#define schar   signed char
#endif
/* shortcuts for well defined 8 bit integer types */

#if USB_CFG_LONG_TRANSFERS_1  /* if more than 254 bytes transfer size required */
#   define usbMsgLen_t_1 unsigned
#else
#   define usbMsgLen_t_1 uchar
#endif
/* usbMsgLen_t is the data type used for transfer lengths. By default, it is
 * defined to uchar, allowing a maximum of 254 bytes (255 is reserved for
 * USB_NO_MSG below). If the usbconfig.h defines USB_CFG_LONG_TRANSFERS to 1,
 * a 16 bit data type is used, allowing up to 16384 bytes (the rest is used
 * for flags in the descriptor configuration).
 */

#define USB_NO_MSG_1  ((usbMsgLen_t_1)-1)   /* constant meaning "no message" */

#ifndef usbMsgPtr_t_1
#define usbMsgPtr_t_1 uchar *
#endif
/* Making usbMsgPtr_t a define allows the user of this library to define it to
 * an 8 bit type on tiny devices. This reduces code size, especially if the
 * compiler supports a tiny memory model.
 * The type can be a pointer or scalar type, casts are made where necessary.
 * Although it's paradoxical, Gcc 4 generates slightly better code for scalar
 * types than for pointers.
 */


USB_PUBLIC void usbInit_1(void);
/* This function must be called before interrupts are enabled and the main
 * loop is entered. We exepct that the PORT and DDR bits for D+ and D- have
 * not been changed from their default status (which is 0). If you have changed
 * them, set both back to 0 (configure them as input with no internal pull-up).
 */
USB_PUBLIC void usbPoll_1(void);
/* This function must be called at regular intervals from the main loop.
 * Maximum delay between calls is somewhat less than 50ms (USB timeout for
 * accepting a Setup message). Otherwise the device will not be recognized.
 * Please note that debug outputs through the UART take ~ 0.5ms per byte
 * at 19200 bps.
 */
extern usbMsgPtr_t_1 usbMsgPtr_1;
/* This variable may be used to pass transmit data to the driver from the
 * implementation of usbFunctionWrite(). It is also used internally by the
 * driver for standard control requests.
 */
USB_PUBLIC usbMsgLen_t_1 usbFunctionSetup_1(uchar data[8]);
/* This function is called when the driver receives a SETUP transaction from
 * the host which is not answered by the driver itself (in practice: class and
 * vendor requests). All control transfers start with a SETUP transaction where
 * the host communicates the parameters of the following (optional) data
 * transfer. The SETUP data is available in the 'data' parameter which can
 * (and should) be casted to 'usbRequest_t *' for a more user-friendly access
 * to parameters.
 *
 * If the SETUP indicates a control-in transfer, you should provide the
 * requested data to the driver. There are two ways to transfer this data:
 * (1) Set the global pointer 'usbMsgPtr' to the base of the static RAM data
 * block and return the length of the data in 'usbFunctionSetup()'. The driver
 * will handle the rest. Or (2) return USB_NO_MSG in 'usbFunctionSetup()'. The
 * driver will then call 'usbFunctionRead()' when data is needed. See the
 * documentation for usbFunctionRead() for details.
 *
 * If the SETUP indicates a control-out transfer, the only way to receive the
 * data from the host is through the 'usbFunctionWrite()' call. If you
 * implement this function, you must return USB_NO_MSG in 'usbFunctionSetup()'
 * to indicate that 'usbFunctionWrite()' should be used. See the documentation
 * of this function for more information. If you just want to ignore the data
 * sent by the host, return 0 in 'usbFunctionSetup()'.
 *
 * Note that calls to the functions usbFunctionRead() and usbFunctionWrite()
 * are only done if enabled by the configuration in usbconfig.h.
 */
USB_PUBLIC usbMsgLen_t_1 usbFunctionDescriptor_1(struct usbRequest *rq);
/* You need to implement this function ONLY if you provide USB descriptors at
 * runtime (which is an expert feature). It is very similar to
 * usbFunctionSetup() above, but it is called only to request USB descriptor
 * data. See the documentation of usbFunctionSetup() above for more info.
 */
#if USB_CFG_HAVE_INTRIN_ENDPOINT_1
USB_PUBLIC void usbSetInterrupt_1(uchar *data, uchar len);
/* This function sets the message which will be sent during the next interrupt
 * IN transfer. The message is copied to an internal buffer and must not exceed
 * a length of 8 bytes. The message may be 0 bytes long just to indicate the
 * interrupt status to the host.
 * If you need to transfer more bytes, use a control read after the interrupt.
 */
#define usbInterruptIsReady_1()   (usbTxLen1 & 0x10)
/* This macro indicates whether the last interrupt message has already been
 * sent. If you set a new interrupt message before the old was sent, the
 * message already buffered will be lost.
 */
#if USB_CFG_HAVE_INTRIN_ENDPOINT3_1
USB_PUBLIC void usbSetInterrupt3_1(uchar *data, uchar len);
#define usbInterruptIsReady3_1()   (usbTxLen3 & 0x10)
/* Same as above for endpoint 3 */
#endif
#endif /* USB_CFG_HAVE_INTRIN_ENDPOINT */
#if USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1    /* simplified interface for backward compatibility */
#define usbHidReportDescriptor_1  usbDescriptorHidReport_1
/* should be declared as: PROGMEM char usbHidReportDescriptor[]; */
/* If you implement an HID device, you need to provide a report descriptor.
 * The HID report descriptor syntax is a bit complex. If you understand how
 * report descriptors are constructed, we recommend that you use the HID
 * Descriptor Tool from usb.org, see http://www.usb.org/developers/hidpage/.
 * Otherwise you should probably start with a working example.
 */
#endif  /* USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH */
#if USB_CFG_IMPLEMENT_FN_WRITE_1
USB_PUBLIC uchar usbFunctionWrite_1(uchar *data, uchar len);
/* This function is called by the driver to provide a control transfer's
 * payload data (control-out). It is called in chunks of up to 8 bytes. The
 * total count provided in the current control transfer can be obtained from
 * the 'length' property in the setup data. If an error occurred during
 * processing, return 0xff (== -1). The driver will answer the entire transfer
 * with a STALL token in this case. If you have received the entire payload
 * successfully, return 1. If you expect more data, return 0. If you don't
 * know whether the host will send more data (you should know, the total is
 * provided in the usbFunctionSetup() call!), return 1.
 * NOTE: If you return 0xff for STALL, 'usbFunctionWrite()' may still be called
 * for the remaining data. You must continue to return 0xff for STALL in these
 * calls.
 * In order to get usbFunctionWrite() called, define USB_CFG_IMPLEMENT_FN_WRITE
 * to 1 in usbconfig.h and return 0xff in usbFunctionSetup()..
 */
#endif /* USB_CFG_IMPLEMENT_FN_WRITE */
#if USB_CFG_IMPLEMENT_FN_READ_1
USB_PUBLIC uchar usbFunctionRead_1(uchar *data, uchar len);
/* This function is called by the driver to ask the application for a control
 * transfer's payload data (control-in). It is called in chunks of up to 8
 * bytes each. You should copy the data to the location given by 'data' and
 * return the actual number of bytes copied. If you return less than requested,
 * the control-in transfer is terminated. If you return 0xff, the driver aborts
 * the transfer with a STALL token.
 * In order to get usbFunctionRead() called, define USB_CFG_IMPLEMENT_FN_READ
 * to 1 in usbconfig.h and return 0xff in usbFunctionSetup()..
 */
#endif /* USB_CFG_IMPLEMENT_FN_READ */

#if USB_CFG_IMPLEMENT_FN_WRITEOUT_1
USB_PUBLIC void usbFunctionWriteOut_1(uchar *data, uchar len);
/* This function is called by the driver when data is received on an interrupt-
 * or bulk-out endpoint. The endpoint number can be found in the global
 * variable usbRxToken. You must define USB_CFG_IMPLEMENT_FN_WRITEOUT to 1 in
 * usbconfig.h to get this function called.
 */
#endif /* USB_CFG_IMPLEMENT_FN_WRITEOUT */

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! NOT MODIFIEDABLE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
 
extern volatile uchar usbTxLen, usbVUSBconfiguration;
extern volatile schar usbRxLen;
extern usbTxStatus_t   usbTxStatus1, usbTxStatus3;
extern uchar usbRxToken, usbCurrentTok, usbNewDeviceAddr, usbDeviceAddr, usbInputBufOffset;
extern uchar usbRxBuf[], usbTxBuf[];


#if USB_CFG_HAVE_FLOWCONTROL_1
extern volatile schar   usbRxLen;
#endif

extern uchar    usbConfiguration;
/* This value contains the current configuration set by the host. The driver
 * allows setting and querying of this variable with the USB SET_CONFIGURATION
 * and GET_CONFIGURATION requests, but does not use it otherwise.
 * You may want to reflect the "configured" status with a LED on the device or
 * switch on high power parts of the circuit only if the device is configured.
 */
#if USB_COUNT_SOF
extern volatile uchar   usbSofCount;
/* This variable is incremented on every SOF packet. It is only available if
 * the macro USB_COUNT_SOF is defined to a value != 0.
 */
#endif
#if USB_CFG_CHECK_DATA_TOGGLING
extern uchar    usbCurrentDataToken;
/* This variable can be checked in usbFunctionWrite() and usbFunctionWriteOut()
 * to ignore duplicate packets.
 */
#endif

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! /NOT MODIFIEDABLE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#endif  /* __ASSEMBLER__ */


/* ------------------------------------------------------------------------- */
/* ----------------- Definitions for Descriptor Properties ----------------- */
/* ------------------------------------------------------------------------- */
/* This is advanced stuff. See usbconfig-prototype.h for more information
 * about the various methods to define USB descriptors. If you do nothing,
 * the default descriptors will be used.
 */

/* all descriptors which may have properties: */
#ifndef USB_CFG_DESCR_PROPS_DEVICE_1
#define USB_CFG_DESCR_PROPS_DEVICE_1                  0
#endif
#ifndef USB_CFG_DESCR_PROPS_CONFIGURATION_1
#define USB_CFG_DESCR_PROPS_CONFIGURATION_1           0
#endif
#ifndef USB_CFG_DESCR_PROPS_STRINGS_1
#define USB_CFG_DESCR_PROPS_STRINGS_1                 0
#endif
#ifndef USB_CFG_DESCR_PROPS_STRING_0_1
#define USB_CFG_DESCR_PROPS_STRING_0_1                0
#endif
#ifndef USB_CFG_DESCR_PROPS_STRING_VENDOR_1
#define USB_CFG_DESCR_PROPS_STRING_VENDOR_1           0
#endif
#ifndef USB_CFG_DESCR_PROPS_STRING_PRODUCT_1
#define USB_CFG_DESCR_PROPS_STRING_PRODUCT_1          0
#endif
#ifndef USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1
#define USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1    0
#endif
#ifndef USB_CFG_DESCR_PROPS_HID_1
#define USB_CFG_DESCR_PROPS_HID_1                     0
#endif
#if !(USB_CFG_DESCR_PROPS_HID_REPORT_1)
#   undef USB_CFG_DESCR_PROPS_HID_REPORT_1
#   if USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1 /* do some backward compatibility tricks */
#       define USB_CFG_DESCR_PROPS_HID_REPORT_1       USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1
#   else
#       define USB_CFG_DESCR_PROPS_HID_REPORT_1       0
#   endif
#endif
#ifndef USB_CFG_DESCR_PROPS_UNKNOWN_1
#define USB_CFG_DESCR_PROPS_UNKNOWN_1                 0
#endif

/* ------------------ forward declaration of descriptors ------------------- */
/* If you use external static descriptors, they must be stored in global
 * arrays as declared below:
 */
#ifndef __ASSEMBLER__
extern
#if !(USB_CFG_DESCR_PROPS_DEVICE_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
char usbDescriptorDevice_1[];

extern
#if !(USB_CFG_DESCR_PROPS_CONFIGURATION_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
char usbDescriptorConfiguration_1[];

extern
#if !(USB_CFG_DESCR_PROPS_HID_REPORT_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
char usbDescriptorHidReport_1[];

extern
#if !(USB_CFG_DESCR_PROPS_STRING_0_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
char usbDescriptorString0_1[];

extern
#if !(USB_CFG_DESCR_PROPS_STRING_VENDOR_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
int usbDescriptorStringVendor_1[];

extern
#if !(USB_CFG_DESCR_PROPS_STRING_PRODUCT_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
int usbDescriptorStringDevice_1[];

extern
#if !(USB_CFG_DESCR_PROPS_STRING_SERIAL_NUMBER_1 & USB_PROP_IS_RAM)
PROGMEM const
#endif
int usbDescriptorStringSerialNumber_1[];

#endif /* __ASSEMBLER__ */

/* ------------------------------------------------------------------------- */
/* ------------------------- Constant definitions -------------------------- */
/* ------------------------------------------------------------------------- */

#if !defined __ASSEMBLER__ && (!defined USB_CFG_VENDOR_ID_1 || !defined USB_CFG_DEVICE_ID_1)
#warning "You should define USB_CFG_VENDOR_ID_1 and USB_CFG_DEVICE_ID_1 in usbconfig1.h"
/* If the user has not defined IDs, we default to obdev's free IDs.
 * See USB-IDs-for-free.txt for details.
 */
#endif

/* make sure we have a VID and PID defined, byte order is lowbyte, highbyte */
#ifndef USB_CFG_VENDOR_ID_1
#   define  USB_CFG_VENDOR_ID_1   0xc0, 0x16  /* = 0x16c0 = 5824 = voti.nl */
#endif

#ifndef USB_CFG_DEVICE_ID_1
#   if USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH_1
#       define USB_CFG_DEVICE_ID_1    0xdf, 0x05  /* = 0x5df = 1503, shared PID for HIDs */
#   elif USB_CFG_INTERFACE_CLASS_1 == 2
#       define USB_CFG_DEVICE_ID_1    0xe1, 0x05  /* = 0x5e1 = 1505, shared PID for CDC Modems */
#   else
#       define USB_CFG_DEVICE_ID_1    0xdc, 0x05  /* = 0x5dc = 1500, obdev's free PID */
#   endif
#endif


#ifndef USB_CFG_HAVE_INTRIN_ENDPOINT3_1
#define USB_CFG_HAVE_INTRIN_ENDPOINT3_1   0
#endif

#endif /* __usbdrv_h_included__ */
